//    File:  Callback.java  -  Demonstrate use of a Callback

public class Callback {

    Callback() {
        System.out.println("In Callback constructor");
        // Send Callee a handle to the Callback object
        Callee hCallee = new Callee(this);
        // Now Callee routines have access to nCount and doubleIt()
        hCallee.doit();
        System.out.println("End of Callback constructor");
    }

    public void doubleIt() {
        nCount += nCount;
    }

    public double getDValue() { return dValue; }

    public void setDValue(double dValue)  { this.dValue = dValue; }

    public static void main(String[] args) {
        System.out.println("\nStarting Callback");
        new Callback();
        System.out.println("End of Callback\n");
    }

    int nCount = 123;        // nCount really should be private...
    private double dValue = 4.56;

}

//    The methods in this class normally would not have access to
//    Callback's variables or methods.  But, since we've passed in
//    a "callback handle" the situation has changed:  it now does

class Callee {
    Callee() {
        System.out.println("Default constructor");
    }

    // Obtain/save access to Callback's variables and methods
    Callee(Callback hCallback) {
        System.out.println("Constructor accepting a (callback) handle");
        this.hCallback = hCallback;
    }

    // Demonstrate that we do have access to nCount and doubleIt()
    void doit() {
        System.out.println("Callee.doit()...");
        // use hCallback to access nCount and doubleIt()
        System.out.println("nCount = " + ++hCallback.nCount);
        hCallback.doubleIt();
        System.out.println("nCount = " + ++hCallback.nCount);

        System.out.println("dValue = " + hCallback.getDValue());
        hCallback.setDValue(5.67);
        System.out.println("dValue = " + hCallback.getDValue());
        
    }

    private Callback hCallback;        // This is the "callback handle"...

}

/* ===== Output =====

Starting Callback
In Callback constructor
Constructor accepting a (callback) handle
Callee.doit()...
nCount = 124
nCount = 249
dValue = 4.56
dValue = 5.67
End of Callback constructor
End of Callback

===== End Output ===== */
